'user strict';

/* Services */

angular.module('myApp.services', [])
.value('version', '0.1')
.factory('SlaveApiService', ['$http', '$q', function($http, $q) {
  var finalUrl = 'slave/index.php/';

  function makeCall(config) {
    config.config = {
      timeout: 7200,
      responseType: "json"
    };
    return $http(config)
    .then(function(response) {
      if (typeof response.data === 'object') {
        return response.data;
      } else {
        // invalid response
        return $q.reject(response.data);
      }
    }, function(response, status, headers, config) {

      // something went wrong
      return $q.reject(response.data);
    });

  }

  function processActiveProducts(response) {

    response.data.data.downloading = 0;

    for(i=0; i<response.data.data.active_products.length; i++) {
      if(typeof response.data.data.active_products[i].download_percent !== "undefined" && response.data.data.active_products[i].downloading) {
        response.data.data.downloading = 1;

        if(response.data.data.active_products[i].download_percent!='-') {
          response.data.data.active_products[i].download_bar_left = response.data.data.active_products[i].download_percent;
        } else {
          response.data.data.active_products[i].download_bar_left = 100;
        }
        break;
      }
      else { 
        response.data.data.active_products[i].download_bar_left = 0;
      }
    }
  }

  return {
    userDetails: null,
    activeProducts: null,
    checkUpdate: function() {
      return makeCall({
        url: finalUrl + 'api/system_updates',
        method: 'GET'
      });
    },
    init: function() {
      return makeCall({
        url: finalUrl + 'api/init',
        method: 'GET'
      });
    },
    doLogin: function(username, password) {
      return makeCall({
        url: finalUrl + 'api/login' +"?username="+username+"&password="+password,
        method: 'GET'
      });
    },
    doLogout: function() {
      return makeCall({
        url: finalUrl + 'api/logout',
        method: 'GET'
      });
    },
    getActiveProducts: function() {
      // return makeCall({
      //   url: finalUrl + 'api/active_products',
      //   method: 'GET'
      // });

      var config = {
        url: finalUrl + 'api/active_products',
        method: 'GET',
        config: {responseType: "json"}
      };

      return $http(config)
      .then(function(response) {
        // adding "downloading" flag
        var i = 0;
        if (typeof response.data === 'object') {
          processActiveProducts(response);
          // response.data.data.downloading = 0;

          // for(i=0; i<response.data.data.active_products.length; i++) {
          //   if(typeof response.data.data.active_products[i].download_percent !== "undefined" && response.data.data.active_products[i].downloading) {
          //     response.data.data.downloading = 1;

          //     if(response.data.data.active_products[i].download_percent!='-') {
          //       response.data.data.active_products[i].download_bar_left = response.data.data.active_products[i].download_percent;
          //     } else {
          //       response.data.data.active_products[i].download_bar_left = 100;
          //     }
          //     break;
          //   }
          //   else { 
          //     response.data.data.active_products[i].download_bar_left = 0;
          //   }
          // }
          return response.data;
        } else {
          // invalid response
          return $q.reject(response.data);
        }
      }, function(response) {
        // something went wrong
        return $q.reject(response.data);
      });
    },
    doSync: function() {
      // return makeCall({
      //   url: finalUrl + 'api/sync',
      //   method: 'GET'
      // });

      var config = {
        url: finalUrl + 'api/sync',
        method: 'GET',
        config: {responseType: "json"}
      };

      return $http(config)
      .then(function(response) {
        // adding "downloading" flag
        var i = 0;
        if (typeof response.data === 'object') {
          processActiveProducts(response);
          // response.data.data.downloading = 0;

          // for(i=0; i<response.data.data.active_products.length; i++) {
          //   if(typeof response.data.data.active_products[i].download_percent !== "undefined" && response.data.data.active_products[i].downloading) {
          //     response.data.data.downloading = 1;

          //     if(response.data.data.active_products[i].download_percent!='-') {
          //       response.data.data.active_products[i].download_bar_left = response.data.data.active_products[i].download_percent;
          //     } else {
          //       response.data.data.active_products[i].download_bar_left = 100;
          //     }
          //     break;
          //   }
          //   else { 
          //     response.data.data.active_products[i].download_bar_left = 0;
          //   }
          // }
          return response.data;
        } else {
          // invalid response
          return $q.reject(response.data);
        }
      }, function(response) {
        // something went wrong
        return $q.reject(response.data);
      });
    },
    checkSync: function() {
      return makeCall({
        url: finalUrl + 'api/sync?check=1',
        method: 'GET'
      });
    },
    startDownload: function(isbn) {
      return makeCall({
        url: finalUrl + 'api/downloaded_products',
        data: {isbn:isbn},
        method: 'POST'
      });
    },
    pause: function(isbn) {
      return makeCall({
        url: finalUrl + 'api/paused_files?isbn='+isbn,
        // data: {isbn:isbn},
        method: 'GET'
      });
    }
  };

}]);

