<?php defined('BASEPATH') OR exit('No direct script access allowed');

require APPPATH.'/libraries/REST_Controller.php';

class Api extends REST_Controller
{
    public function system_updates_get()
    {
        $this->load->library('updater');
        $this->load->library('migration');

        $response = array('update_status' => 0);

        try
        {
            $status_update = $this->updater->check_and_update();

            if($status_update)
            {
                $response['update_status'] = 1;
            }

            log_message('info', 'Update OK');
        }
        catch(Exception $e)
        {
            log_message('error', 'Update: ' . display_exception($e));
        }


        if(!$this->migration->current())
        {
            log_message('error', $this->migration->error_string());
            $response['migration'] = $this->migration->error_string();
            $this->_response_server_error($response);
        }
        else
        {
            log_message('info', 'Migration OK');
            $response['migration'] = 'Migration complete';
            $this->_response_success($response);
        }

    }

/*
 * 
 * name: init_get
 * @param 
 * @return the user details in case that a previous version exists.
 */ 
    public function init_get()
    {
        $this->load->library('slave_sams');
        $this->load->model('user_model');

        $SID = $this->slave_sams->get_local_sid();

        if($SID!==FALSE)
        {
            $user = $this->user_model->get_by_sid($SID);

            if(!empty($user))
            {
                // this is the authenticated user
                $this->_response_success(array('user' => $user));
            }
            else
            {
                $this->_response_authentication_error('Authentication error. Internet connection required for first login.');
            }
        }
        else
        {
            $this->_response_authentication_error('Couldn\'t get a valid session');
        }
    }

/*
 * 
 * name: logout_get
 * destroy a local session
 *  
 */ 
    public function logout_get()
    {
        $this->load->library('sams');
        $this->load->library('slave_sams');
    //     $this->load->model('user_model');
        $this->load->model('session_model');

        $SID = $this->slave_sams->get_local_sid();

        $this->session_model->delete($SID);

        $this->sams->remove_cookie();

        $this->_response_success();
    }

    public function login_get()
    {
        log_message('info', 'Api::login_get');

        $this->load->library('sams');
        $this->load->library('slave_sams');
        $this->load->model('user_model');
        $this->load->model('session_model');

        $username = $this->get('username');
        $password = $this->get('password');

        $user = $this->user_model->get_by(array(
            'username' => $username,
            'password' => $password,
        ));

        // local login ok?
        if(!empty($user))
        {
            // get local session and write cookie. Si no hay cookie, obtener su valor desde la tabla sessions, porque sino puede dar error el soap
            $SID = $this->slave_sams->get_local_sid();

            log_message('info', 'Local SID: ' . $SID);

            $this->session_model->delete($SID);

            $this->session_model->insert(array(
                'sams_id' => $user->sams_id,
                'session_id' => $SID
            ));

            $this->_response_success(array(
                'user' => $user
            ));
        }
        else // try remote login
        {
            // get remote sid
            $SID = $this->sams->get_valid_sid();

            if(!empty($SID))
            {
                try
                {
                    $SAMS_ID = $this->sams->login(array(
                        'SID' => $SID,
                        'username' => $username,
                        'password' => $password
                    ));

                    if(!empty($SAMS_ID))
                    {
                        // TODO: show user details
                        // get user details
                        // $user_details = $this->sams->get_user_details($SID);
                        // create user from user details
                        // $this->user_model->insert_from_response($user_details);

                        // create user
                        $this->user_model->insert(array(
                            'username' => $username,
                            'password' => $password,
                            'sams_id' => $SAMS_ID
                        ));

                        // create a clean session for this user
                        $this->session_model->delete($SID);
                        $this->session_model->insert(array(
                            'sams_id' => $SAMS_ID,
                            'session_id' => $SID
                        ));

                        $this->_response_success(array(
                            'user' => $this->user_model->get($SAMS_ID)
                        ));
                    }
                    else
                    {
                        $this->_response_authentication_error('Authentication error. Internet connection required for first login.');
                    }
                }
                catch(Exception $e)
                {
                    $this->_response_server_error(display_exception($e));
                }
            }
            else
            {
                $this->_response_authentication_error('Authentication error. Internet connection required for first login.');
            }
        }

    }

    // public function login0_get()
    // {
    //     $this->load->model('user_model');
    //     $this->load->model('session_model');

    //     $username = $this->get('username');
    //     $password = $this->get('password');

    //     try
    //     {
    //         // get a valid local session 
    //         $SID = $this->slave_sams->get_session_id();

    //         // if there is not any, the make a local one
    //         if(empty($SID))
    //         {
    //             $SID = $this->config->item('default_local_sid');
    //         }

    //         if($SID!==FALSE)
    //         {
    //             // search a local user
    //             $user = $this->user_model->get_by_sid(array(
    //                 'session_id' => $SID, 
    //                 'username' => $username, 
    //                 'password' => $password
    //             )); 

    //             if(!empty($user))
    //             {
    //                 $this->_response_success(array('user' => $user));
    //             }
    //             else
    //             {
    //                 // Try to get a valid remote session

    //                 $SID = $this->slave_sams->get_session_id(TRUE);

    //                 log_message('info', 'SID:'.$SID);

    //                 // Login against server or local(if $SID is empty)
    //                 $SAMS_ID = $this->slave_sams->login($SID, $username, $password);

    //                 if($SAMS_ID!==FALSE)
    //                 {
    //                     // does the user exist?
    //                     $user = $this->user_model->get($SAMS_ID);

    //                     if(empty($user))
    //                     {
    //                         $user = new stdClass();
    //                         $user->username = $username;
    //                         $user->password = $password;
    //                         $user->sams_id = $SAMS_ID;
    //                         $this->user_model->insert($user);
    //                     }
    //                     // if the user does exist, check if the password needs to be updated
    //                     elseif($user->password!=$password)
    //                     {
    //                         $user->password = $password;
    //                         $this->user_model->update($user);
    //                     }

    //                     // create local session
    //                     try
    //                     {
    //                         $d = new \DateTime();

    //                         $this->session_model->delete_by(array(
    //                             'expires_at <' => $d->format('Y-m-d H:i:s'),
    //                             'sams_id' => $user->sams_id
    //                         ));

    //                         $session = $this->session_model->get($SID);

    //                         if(empty($session))
    //                         {
    //                             $session = new stdClass();
    //                             $session->session_id = $SID;
    //                             $session->sams_id = $user->sams_id;
    //                             $d->modify('+ ' . COOKIE_TIMESTAMP . ' seconds');
    //                             $session->expires_at = $d->format('Y-m-d H:i:s');
    //                             $this->session_model->insert($session);
    //                         }
    //                     }
    //                     catch(Exception $e)
    //                     {
    //                         log_message('error', display_exception($e));
    //                     }

    //                     $this->_response_success(array('user' => $user));
    //                 }
    //                 else
    //                 {
    //                     $this->_response_authentication_error();
    //                 }
    //             }
    //         }
    //         else
    //         {
    //             $this->_response_server_error('Could get a valid SID');
    //         }
    //     }
    //     catch(Exception $e)
    //     {
    //         $this->_response_server_error(display_exception($e));
    //     }
    // }

    public function sync_get()
    {
        $this->load->library('sams');
        $this->load->library('slave_sams');
        $this->load->model('request_model');

        try
        {
            $check = $this->get('check');

            if(empty($check))
            {
                $output = array();

                $session = $this->slave_sams->authenticate_online();

                // sync requests
                $sync_requests = $this->slave_sams->sync_requests($output, $session);

                log_message('info', 'Requests synced');

                // sync products
                $active_products = $this->slave_sams->sync_products($session);

                log_message('info', 'Products synced');

                $this->_response_success(array(
                    'active_products' => $active_products,
                    'sync_requests' => $output,
                ));
            }
            else
            {
                $SID = $this->slave_sams->get_local_sid();

                $session = $this->session_model->get($SID); 

                $this->_response_success(array(
                    'qty_requests' => $this->request_model->count_by(array(
                        'sams_id' => $session->sams_id,
                        'synced_at IS NULL' => NULL
                    ))
                ));
            }
        }
        catch(Exception $e)
        {
            log_message('error', 'Exception (api::sync_get)');
            log_message('error', display_exception($e));
            $this->_response_server_error('There was an error. Please check your connection.');
        }
    }

    public function active_products_get()
    {
        $this->load->library('slave_sams');
        $this->load->library('package_downloader');
        $this->load->model('session_model');
        $this->load->model('product_model');

        try
        {

            $local_SID = $this->slave_sams->get_local_sid();

            if(!empty($local_SID))
            {
                $session = $this->session_model->get($local_SID);

                if(!empty($session))
                {
                    $active_products = $this->product_model->get_by_user($session->sams_id, array(
                        'prefix_local_path' => $this->config->item('products_folder')
                    ));

                    $this->package_downloader->add_file_status($active_products);

                    $this->_response_success(array(
                        'active_products' => $active_products
                    ));
                }
                else
                {
                    $this->_response_authorization_error('User session not found');
                }
            }
            else
            {
                $this->_response_authorization_error('Local session id not found');
            }
        }
        catch(Exception $e)
        {
            $this->_response_server_error(display_exception($e));
        }
    }

    public function downloaded_products_post()
    {
        $this->load->library('package_downloader','slave_sams');
        $this->load->model('product_model');

        $isbn = $this->post('isbn');

        try
        {
            $active_products = $this->slave_sams->sync_products();

            $i=0;
            while($i<count($active_products) && $active_products[$i]->isbn!=$isbn) $i++;

            if($i<count($active_products) && $active_products[$i]->isbn==$isbn)
            {
                $ap = $active_products[$i];

                $this->package_downloader->download_package($ap->zip_file, $this->config->item('products_folder') . $isbn);

                $res = $this->package_downloader->decompress_package($ap->zip_file, $this->config->item('products_folder') . $isbn);
                if(!empty($res))
                {
                    // if there is a manifest, change Markbook address (this process vary between book collections)
                    $this->package_downloader->set_markbook_address($isbn);

                    $this->package_downloader->update_files($isbn);

                    $this->product_model->update($isbn, array(
                        'downloaded_version' => $ap->version
                    ));

                    $this->_response_success();
                }
                else
                {
                    $this->_response_server_error('Error while decompressing package');
                }

            }
            else
            {
                $this->_response_authorization_error('There is no license to download this product');
            }


        }
        catch(Exception $e)
        {
            if(strpos($e->getMessage(), 'Failed writing body')!==FALSE)
            {
                $this->_response_success(array(
                    'isbn' => $isbn,
                    'status' => 'paused'
                ));
            }
            else
            {
                $this->_response_server_error(display_exception($e));
            }
        }
    }

    public function paused_files_get()
    {
        $this->load->library('slave_sams');
        $SID = $this->slave_sams->get_local_sid();

        if(!empty($SID))
        {
            try
            {
                $this->load->library('package_downloader');

                $response = $this->package_downloader->pause_download($this->get('isbn'));

                if($response)
                {
                    $this->_response_success();
                }
                else
                {
                    $this->_response_server_error('Pause wasn\'t set');
                }
            }
            catch(Exception $e)
            {
                $this->_response_server_error(display_exception($e));
            }
        }
        else
        {
            $this->_response_authentication_error();
        }
    }

    public function exe_get()
    {
        $this->load->library('slave_sams');

        try
        {
            $active_products = $this->slave_sams->save_request();

            $this->_response_success(array(
                'active_products' => $active_products,
            ));
        }
        catch(Exception $e)
        {
            $this->_response_server_error(display_exception($e));
        }
    }

    public function reset_database_get()
    {
        $tables = array('migrations','portable_requests','portable_products_x_users','portable_products','portable_sessions','portable_users');

        $this->db->query('SET FOREIGN_KEY_CHECKS=0');

        foreach($tables as $t)
        {
            $this->db->truncate($t);
        }

        $this->db->query('SET FOREIGN_KEY_CHECKS=1');
    }
}

