<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed');

class Markbook extends CI_Controller {

/*
 * 
 * name: _send_to_server
 * @param input $final_url
 * @param input $SID
 * @param output $resp
 * @param output $content_type
 * @return 
 */ 
    private function _send_to_server($final_url, $SID, &$response, &$content_type)
    {
        // Get valid remote session
        $session = $this->slave_sams->authenticate_online();
        $SID = $session->session_id;

        $ch = curl_init($final_url);
        curl_setopt($ch, CURLOPT_COOKIE, 'MEC_WS_SESSIONID='.$SID);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);

        $response = curl_exec($ch);
        $info = curl_getinfo($ch);
        $content_type = $info['content_type'];

        $curl_error = curl_error($ch);
        $curl_errno = curl_errno($ch);

        curl_close($ch);

        if($curl_errno)
        {
            throw new Exception($curl_error);
        }

        return TRUE;
    }

    // TODO: Test the following cases with and without connection
    // 1) trying to open markbook
    // 2) opening an exercise
    // 3) resolving an exercise
    // 4) listing notes
    // 5) adding a note
    // 6) editing a note
    // 7) deleting a note
    // 8) cases 4 to 7 for links

	public function index()
	{
        $this->load->helper('url');
        $this->load->library('output');
        $this->load->library('slave_sams');
        $this->load->library('uri');
        $this->load->model('session_model');
        $this->load->model('request_model');
        $this->load->model('user_model');
        $this->load->model('product_model');

        $response = '';
        $content_type = '';
        $method = strtolower($this->input->get('method'));
        $callback = $this->input->get('callback');

        // is it an api request o the user is trying to open markbook ?
        $is_api_request = strpos($this->uri->uri_string(), 'cl_scoserver.php')!==FALSE
            || strpos($this->uri->uri_string(), 'usernotes')!==FALSE
            || strpos($this->uri->uri_string(), 'userlinks')!==FALSE
            || strpos($this->uri->uri_string(), 'checkuser')!==FALSE
            ;

        try // global, to prevent unexpected errors
        {
            // build markbook url and final url
            
            // NOTE: this array starts from 1 !
            $uri = $this->uri->segment_array();
            $uri = array_values($uri);

            // get markbook url from manifest
            $pos = array_search('imsmanifest.xml', $uri);
            $offset = 2; // to exclude class name and method name
            if($pos !== FALSE)
            {
                $length = $pos+1-$offset; // determine how many elements should be included from $offset to $pos
                $manifest_path = implode('/', array_slice($uri, $offset, $length));
                $simple_xml = simplexml_load_file($this->config->item('products_folder') . $manifest_path);
                $markbook_url = $simple_xml->organizations->organization->metadata->Markbook->attributes()->originalUrl;
            }
            else
            {
				$pos = 1;
                $length = $pos+1-$offset; // determine how many elements should be included from $offset to $pos
                // TODO esta ruta tiene que tomarse del libro
                $markbook_url = "http://tiger.macmillan.es/markbook/";
            }

            // Build real url
            $final_url = $markbook_url;

            // add rest of the url
            if(!empty($uri[$pos+1]))
            {
                $final_url .= implode('/', array_slice($uri, $pos+1));
            }

            // Get params
            $get = $this->input->get();

            $params = '';
            if(is_array($get)&&count($get)>0) $params = '?' . http_build_query($get);

            try
            {

                // if there is connection, send the request to the server (markbook or api)

                $session = $this->slave_sams->authenticate_online();
                $SID = $session->session_id;

                if($is_api_request) // send request to the remote api
                {
                    $this->_send_to_server($final_url.$params, $SID, $response, $content_type);

                    $output = $response;
                    // $output = !empty($callback)
                    //     ? $callback . "(" . $response . ")" 
                    //     : $response;

                    $this->output
                        ->set_status_header(200)
                        ->set_content_type($content_type)
                        ->set_output($output);
                }
                else // assume a user opening markbook
                {
                    $markbook_url = rtrim($markbook_url, '/');

                    redirect($markbook_url . '/php/api/save_session?MEC_WS_SESSIONID=' . $SID . '&redirect_url=' . urlencode($final_url.$params));
                }
            }
            catch(Exception $e)
            {
                // the user is not connected

                if($is_api_request)
                {
                    $SID = $this->slave_sams->get_local_sid();
                    $session = $this->session_model->get($SID);
                    $content_type = 'application/json';

                    if(strpos($this->uri->uri_string(), 'cl_scoserver.php')!==FALSE)
                    {
                        $json = $this->input->get('json');

                        if($method=='set')
                        {
                            $insert_id = $this->request_model->insert(array(
                                'sams_id' => $session->sams_id,
                                'url' => $final_url.$params,
                                'json' => $json,
                                'type' => 'exercise_result',
                            ));

                            $response = json_encode(array(
                                'result' => TRUE,
                                'require_userdata' => 0,
                            ));
                        }
                        else
                        {
                            $this->load->helper('array');
                            $params = json_decode($json, TRUE);
                            $params = array_filter(elements(array('package_identifier','item_identifier', 'book_id', 'element_id'), $params));
                            $attempts = $this->request_model->count_by($params);

                            $request = $this->request_model->order_by('created_at', 'desc')->limit(1)->get_by($params);

                            if($attempts>0)
                            {
                                // $arr = json_decode($request->json, TRUE);

                                // send a dummy "data" with the qty of attempts
                                $response = json_encode(array(
                                    'result' => array(
                                        // 'data' => $arr['data'], // '[true]',
                                        'data' => $request->exercise_data,
                                        'score' => $request->score,
                                        'total' => $request->total,
                                        'attempts' => $attempts,
                                    ),
                                ));
                            }
                            else
                            {
                                $response = json_encode(array(
                                    'result' => array(),
                                ));
                            }
                        }
                    }
                    elseif(strpos($this->uri->uri_string(), 'usernotes_init')!==FALSE)
                    {
                        $response = json_encode(array(
                            'notes' => array(),
                            'links' => array(),
                        ));
                    }
                    elseif(strpos($this->uri->uri_string(), 'userlinks')!==FALSE||strpos($this->uri->uri_string(), 'usernotes')!==FALSE)
                    {
                        if($method=='add')
                        {
                            $insert_id = $this->request_model->insert(array(
                                'sams_id' => $session->sams_id,
                                'url' => $final_url . $params,
                                'type' => 'usernote',
                            ));

                            $response = json_encode(array(
                                'id' => $insert_id
                            ));
                        }
                        else
                        {
                            $response = json_encode(array(
                                'error' => 'Connection error',
                                'errordb' => 'Connection error',
                            ));
                        }
                    }
                    elseif(strpos($this->uri->uri_string(), 'checkuser')!==FALSE)
                    {
                        $user = $this->user_model->get_by_sid($SID);
                        $products = $this->product_model->get_by_user($session->sams_id, array(
                            'prefix_local_path' => $this->config->item('products_folder')
                        ));
                        $active_products = array();
                        for ($i=0; $i < count($products); $i++) { 
                           $active_products[] = $products[$i]->isbn;
                        }
                        
                        $response = json_encode(array(
                                'user1'=>$user,
                                'products'=>$products,
                                'require_userdata'=> 0,
                                'result'=> true,
                                'user'=> array(
                                    'extra' => array(
                                        'active_products'=> $active_products,
                                        'message'=> "noerror.",
                                        'status'=> "0"
                                    ),
                                    'group_code'=> 1,
                                    'id'=> $user->sams_id,
                                    'name'=> $user->username
                                )
                            ));
                    $output = !empty($callback)
                        ? $callback . "(" . $response . ")" 
                        : $response;

                    $this->output->set_content_type($content_type)
                        ->set_output($output);
                    }
                    else
                    {
                        $response = 0;
                    }

                    $output = !empty($callback)
                        ? $callback . "(" . $response . ")" 
                        : $response;

                    $this->output->set_content_type($content_type)
                        ->set_output($output);
                }
                else // the user is trying to open markbook
                {
                    $this->load->view('markbook_offline');
                }
            }

            ///// if(strpos($this->uri->uri_string(), 'cl_scoserver.php')!==FALSE)
            ///// {

            /////     // saving / retrieving exercises
            /////     try
            /////     {
            /////     }
            /////     catch(Exception $e)
            /////     {
            /////         // without connection
            /////         $content_type = 'application/json';

            /////         if(strtolower($get['method'])=='set')
            /////         {
            /////             $SID = $this->slave_sams->get_local_sid();
            /////             $session = $this->session_model->get($SID);

            /////             $this->request_model->insert(array(
            /////                 'sams_id' => $session->sams_id,
            /////                 'url' => $final_url
            /////             ));

            /////             $response = json_encode(array(
            /////                 'result' => TRUE,
            /////                 'require_userdata' => 0,
            /////             ));
            /////         }
            /////         else
            /////         {
            /////             $response = json_encode(array(
            /////                 'result' => array(),
            /////             ));
            /////         }
            /////         $resp = !empty($get['callback'])
            /////             ? $get['callback'] . "(" . $response . ")" 
            /////             : $response;
            /////     }

            /////     $this->output->set_content_type($content_type)
            /////         ->set_output($resp);
            ///// }
            ///// elseif(strpos($this->uri->uri_string(), 'usernotes_init')!==FALSE)
            ///// {
            /////     try
            /////     {
            /////         try
            /////         {
            /////             // with connection
            /////             $this->_send_to_server($final_url, $resp, $content_type);
            /////         }
            /////         catch(Exception $e)
            /////         {
            /////             $this->load->model('session_model');
            /////             $this->load->model('request_model');

            /////             $content_type = 'application/json';

            /////             if(strtolower($get['method'])=='get')
            /////             {
            /////                 $response = json_encode(array(
            /////                     'notes' => array(),
            /////                     'links' => array(),
            /////                 ));
            /////             }
            /////             else
            /////             {
            /////                 $SID = $this->slave_sams->get_local_sid();
            /////                 $session = $this->session_model->get($SID);

            /////                 $this->request_model->insert(array(
            /////                     'sams_id' => $session->sams_id,
            /////                     'url' => $final_url
            /////                 ));

            /////                 $response = json_encode(array(
            /////                     'result' => TRUE,
            /////                     'require_userdata' => 0,
            /////                 ));
            /////             }

            /////             $resp = !empty($get['callback'])
            /////                 ? $get['callback'] . "(" . $response . ")" 
            /////                 : $response;
            /////         }
            /////     }
            /////     catch(Exception $e)
            /////     {
            /////         $content_type = 'application/json';
            /////         $resp = display_exception($e);
            /////     }

            /////     $this->output->set_content_type($content_type)
            /////         ->set_output($resp);
            ///// }
            ///// else // opening markbook
            ///// {
            /////     // try
            /////     // {
            /////     //     // Get valid remote session
            /////     //     $session = $this->slave_sams->authenticate_online();

            /////     //     $SID = $session->session_id;

            /////     //     $markbook_url = rtrim($markbook_url, '/');

            /////     //     redirect($markbook_url . '/php/api/save_session?MEC_WS_SESSIONID=' . $SID . '&redirect_url=' . urlencode($final_url));

            /////     // }
            /////     // catch(Exception $e)
            /////     // {
            /////     $this->load->view('markbook_offline');
            /////     // }
            ///// }
        }
        catch(Exception $e)
        {
            $content_type = 'application/json';
            $resp = display_exception($e);

            $this->output->set_content_type($content_type)
                ->set_output($resp);
        }

	}
}

