<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');

class MY_DB_mysql_driver extends CI_DB_mysql_driver {

    final public function __construct($params)
    {
        parent::__construct($params);
    }

    /**
     * Insert Ingore
     *
     * What this basically does is compile a query as an INSERT and then simply
     * amend the string "INSERT" at the start of the query to be "INSERT IGNORE".
     * 
     * INSERT IGNORE means that any errors for duplicate rows will be suppressed to
     * warnings. 
     * 
     * @access    public
     * @param    string    the table name
     * @param    string    optional insert data
     * @return    object
     */
    function insert_ignore($table, $data = NULL)
    {
        if(!empty($data)) $this->set($data);

        $query_string = 'INSERT IGNORE'.substr($this->insert_string($table,$data), 6);

        $this->query($query_string);

        $this->_reset_write();

        return TRUE;
    }

    function on_duplicate_update($table, $insert_values = NULL, $update_fields = NULL, $options = array())
    {
        $fields = array_keys($insert_values);
        $values = array_values($insert_values);

        $update_values = array();

        if(is_null($update_fields))
        {
            return FALSE;
        }

        foreach($values as $k => $v)
        {
            $values[$k] = $this->escape($v);
        }

        foreach($update_fields as $f => $v)
        {
            $update_values[] = $f.'=' . (!is_null($v)?$this->escape($v):'NULL');
        }

        /*
         * Useful to set the last insert id on the fields specified, even when there are not new data
         * http://dev.mysql.com/doc/refman/5.0/en/information-functions.html#function_last-insert-id
         */
        if(isset($options['set_last_insert_id']))
        {
            $update_values[] = $options['set_last_insert_id'].'=LAST_INSERT_ID('.$options['set_last_insert_id'].')';
        }

        // $marks = array_fill(0,count($fields), '?');

        //$q = "INSERT INTO ".$table."(" . implode(",", $fields) . ") VALUES (" . implode(",", $marks) . ") ON DUPLICATE KEY UPDATE " . implode(",", $update_values);
        $q = "INSERT INTO ".$table."(" . implode(",", $fields) . ") VALUES (" . implode(",", $values) . ") ON DUPLICATE KEY UPDATE " . implode(",", $update_values);

        //return $this->query($q, $values);
        $this->query($q);

        $this->_reset_write();

        return TRUE;
    }

    /** 
     * This function will allow you to do complex group where clauses in to c and (a AND b) or ( d and e)
     * This function is needed as else the where clause will append an automatic AND in front of each where Thus if you wanted to do something
     * like a AND ((b AND c) OR (d AND e)) you won't be able to as the where would insert it as a AND (AND (b...)) which is incorrect. 
     * Usage: start_group_where(key,value)->where(key,value)->close_group_where() or complex queries like
     *        open_bracket()->start_group_where(key,value)->where(key,value)->close_group_where()
     *        ->start_group_where(key,value,'','OR')->close_group_where()->close_bracket() would produce AND ((a AND b) OR (d))
     * @param $key mixed the table columns prefix.columnname
     * @param $value mixed the value of the key
     * @param $escape string any escape as per CI
     * @param $type the TYPE of query. By default it is set to 'AND' 
     * @return db object.  
     */
    function start_group_where($key,$value=NULL,$escape,$type="AND")
    {
        $this->open_bracket($type); 
        return parent::_where($key, $value,'',$escape); 
    }

    /**
     * Strictly used to have a consistent close function as the start_group_where. This essentially callse the close_bracket() function. 
     */
    function close_group_where()
    {
        return $this->close_bracket();  
    }

    /**
     * Allows to place a simple ( in a query and prepend it with the $type if needed. 
     * @param $type string add a ( to a query and prepend it with type. Default is $type. 
     * @param $return db object. 
     */
    function open_bracket($type="AND")
    {
        if(count($this->ar_where)>0)
        {
            $this->ar_where[] = $type . " (";
        }
        else
        {
            $this->ar_where[] = " (";
        }
        return $this;  
    }   

    /**
     * Allows to place a simple ) to a query. 
     */
    function close_bracket()
    {
        $this->ar_where[] = ")"; 
        return $this;       
    }
}

