<?php

class Package_downloader {

    protected $download_status_file;
    protected $uncompress_status_file;
    protected $pausing_file;
    protected $part_size = 0;

    public function __construct()
    {
        $this->ci =& get_instance();
    }

    public function update_products($soap_products)
    {
        foreach($soap_products as $p)
        {
            $product_folder = $this->ci->config->item('products_folder') . $p['isbn'] . '/';

            if(!is_dir($product_folder))
            {
                log_message('info', 'Creating product folder ' . $product_folder);
                mkdir(rtrim($product_folder, '/'));
            }

            // $this->download_status_file = $product_folder . 'download_status.txt';

            $file = $product_folder . basename($p['image']);
            if(!file_exists($file))
            {
                $this->_download($p['image'], $file, FALSE);
            }
        }
    }

    public function download_package($url, $destination_folder)
    {
        // TODO: test communication trainer products
        // TODO: add interface/tiger products
        // TODO: resume downloads

        $destination_folder = rtrim($destination_folder, '/') . '/';

        // package to be downloaded
        $pkg_file = $destination_folder . basename($url);

        // current md5 file
        $md5_file = $pkg_file . '.md5';

        // md5 to be downloaded
        $new_md5_file = $pkg_file . '.new.md5';

        $this->pausing_file = $destination_folder . 'pausing.txt';
        $this->download_status_file = $destination_folder . 'download_status.txt';
        $this->uncompress_status_file = $destination_folder . 'uncompress_status.txt';

        if(file_exists($this->pausing_file))
        {
            unlink($this->pausing_file);
        }

        file_put_contents($this->download_status_file, '0;Getting md5');

        // download md5
        if(file_exists($new_md5_file)) unlink($new_md5_file);
        $this->_download($url . '.md5', $new_md5_file);

        $new_md5_data = substr(file_get_contents($new_md5_file), 0, 32);

        if(file_exists($pkg_file))
        {
            file_put_contents($this->download_status_file, '0;Checking integrity');

            $md5_data = md5_file($pkg_file);

            log_message('info', 'Zip already exists');

            // if the file does exist and has the same md5 as online, then it is ok
            if($new_md5_data==$md5_data)
            {
                log_message('info', 'Zip is ok');

                // update current md5 file
                file_put_contents($md5_file, $md5_data);

                // update modification time
                touch($pkg_file);

                return TRUE;
            }
            else
            {
                unlink($pkg_file);
            }
        }

        // else download file
        $part_file = $pkg_file . '.part';
        $part_status = FALSE;

        if(file_exists($part_file))
        {
            if(file_exists($md5_file))
            {
                $md5_data = file_get_contents($md5_file);

                $part_status = $new_md5_data==$md5_data;
            }

            if(empty($part_status))
            {
                // start a new download. truncate part file
                file_put_contents($md5_file, $new_md5_data);
                unlink($part_file);
            }
        }

        $this->_download($url, $part_file, TRUE, TRUE);

        log_message('info', 'Rename ' . $part_file . ' to ' . $pkg_file);

        // verify integrity
        if(!rename($part_file, $pkg_file))
        {
            return FALSE;
        }

        file_put_contents($this->download_status_file, '100;Checking integrity');

        $md5_data = md5_file($pkg_file);

        log_message('info', 'MD5 ok: ' . $new_md5_data);
        log_message('info', 'MD5 file: ' . $md5_data);

        // verify file
        if(!($new_md5_data==$md5_data)) 
        {
            unlink($pkg_file);
            throw new Exception('File is corrupted');
        }

        return TRUE;
    }

    public function decompress_package($url, $destination_folder)
    {

        $this->ci->load->library('my_pclzip');

        touch($this->uncompress_status_file);

        $destination_folder = rtrim($destination_folder, '/') . '/';

        $zip = $destination_folder . basename($url);

        try
        {
            $extractor = $this->ci->my_pclzip->extract($zip, $destination_folder);

            if(file_exists($this->uncompress_status_file)) unlink($this->uncompress_status_file);
        }
        catch(Exception $e)
        {
            log_message('error', display_exception($e));
            if(file_exists($this->uncompress_status_file)) unlink($this->uncompress_status_file);
            throw $e;
        }

        return !empty($extractor);

    }

    public function add_file_status(&$active_products)
    {

        foreach($active_products as $k => $p)
        {
            clearstatcache();

            $download_status_file = $this->ci->config->item('products_folder') . $p->isbn . '/download_status.txt';
            $uncompress_status_file = $this->ci->config->item('products_folder') . $p->isbn . '/uncompress_status.txt';
            $zip_file = $this->ci->config->item('products_folder') . $p->isbn . '/' . basename($p->zip_file);

            $active_products[$k]->has_version_web = (int)file_exists($this->ci->config->item('products_folder') . $p->isbn . '/index.html');
            if($active_products[$k]->has_version_web === 0) {
                if(file_exists($this->ci->config->item('products_folder') . $p->isbn . '/ebook/index.html'))
                {
                    $active_products[$k]->has_version_web = 2;
                    $active_products[$k]->index_string = 'ebook/';
                }
            }
            else
            {
                $active_products[$k]->index_string = '?package=sources/';
            }

            // does exist download_status.txt ?
            $has_download_status = file_exists($download_status_file);

            // does exist uncompress_status.txt ?
            $has_uncompress_status = file_exists($uncompress_status_file);

            $downloading = FALSE;

            if($has_download_status)
            {
                $chk = $download_status_file.'.check';
                touch($chk);
                $now = new \DateTime();
                $now->setTimestamp(filemtime($chk));

                $from_file = new \DateTime();
                $from_file->setTimestamp(filemtime($download_status_file));

                unlink($chk);

                $downloading = abs($now->getTimestamp() - $from_file->getTimestamp())<60;

            }

            $uncompressing = FALSE;
            if($has_uncompress_status&&file_exists($zip_file))
            {
                $chk = $zip_file.'.check';
                $from_zip_file1 = new \DateTime();
                $from_zip_file1->setTimestamp(filemtime($zip_file));

                $from_zip_file2 = new \DateTime();
                $from_zip_file2->setTimestamp(filemtime($uncompress_status_file));

                touch($chk);
                $now = new \DateTime();
                $now->setTimestamp(filemtime($chk));

                unlink($chk);
                $max_time = 900; // ini_get("max_execution_time");

                $uncompressing = (abs($now->getTimestamp() - $from_zip_file1->getTimestamp())<$max_time) && (abs($now->getTimestamp() - $from_zip_file2->getTimestamp())<$max_time); 
            }

            $active_products[$k]->downloading = (int)($downloading || $uncompressing);

            $active_products[$k]->download_percent = 0;
            $active_products[$k]->download_legend = '';

            if(!empty($downloading))
            {
                $download_status = file_get_contents($download_status_file);

                if(empty($download_status))
                {
                    usleep(200000);
                    $download_status = file_get_contents($download_status_file);
                }

                if(!empty($download_status))
                {
                    list($download_percent, $download_legend) = explode(';', $download_status);
                    $active_products[$k]->download_percent = $download_percent;
                    $active_products[$k]->download_legend = $download_legend;
                }
                else
                {
                    $active_products[$k]->download_percent = '-';
                    $active_products[$k]->download_legend = 'Checking ...';
                }
            }
            elseif(!empty($uncompressing))
            {
                $active_products[$k]->download_percent = '99';
                $active_products[$k]->download_legend = 'Deploying ...';
            }
        }
    }

    private function _download($url, $destination, $show_progress = FALSE, $use_write_callback = FALSE)
    {
        $fp_curl_log = fopen(APPPATH . 'logs' . DIRECTORY_SEPARATOR . 'curl.log', 'w');

        $ch = curl_init($url);

        $this->part_size = 0;

        if(file_exists($destination))
        {
            // resume download
            $this->part_size = filesize($destination);
            curl_setopt($ch, CURLOPT_RESUME_FROM, $this->part_size);

            // append to part file
            $this->fp = fopen($destination, "a");
        }
        else
        {
            $this->fp = fopen($destination, 'w');
        }
        
        if($show_progress)
        {
            // PROGRESS BAR
            curl_setopt($ch, CURLOPT_NOPROGRESS, FALSE);    
            curl_setopt($ch, CURLOPT_PROGRESSFUNCTION, array($this, '_upload_callback'));  
        }

        curl_setopt($ch, CURLOPT_VERBOSE, 1);

        curl_setopt($ch, CURLOPT_STDERR, $fp_curl_log);

        curl_setopt($ch, CURLOPT_FILE, $this->fp);

        // SET TIMEOUT (seconds)
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 20);    

        // curl_setopt($ch, CURLOPT_TIMEOUT, 30);    

        curl_setopt($ch, CURLOPT_LOW_SPEED_LIMIT, 1);    
        curl_setopt($ch, CURLOPT_LOW_SPEED_TIME, 20);    

        if($use_write_callback)
        {
            curl_setopt($ch, CURLOPT_WRITEFUNCTION, array($this, '_write_callback'));
        }

        curl_exec($ch);

        $curl_error = curl_error($ch);

        $curl_info = curl_getinfo($ch);

        curl_close($ch);

        fclose($this->fp);

        fclose($fp_curl_log);

        if($show_progress)
        {
            unlink($this->download_status_file);
        }

        if(!empty($curl_error))
        {
            log_message('error', 'curl_getinfo: ' . print_r($curl_info, TRUE));
            throw new Exception('Download failed: ' . $curl_error);
        }

        return TRUE;
    }

    public function _upload_callback($total_down, $downloaded, $total_up, $uploaded)
    {
        $download_status = $total_down>0 
            ? round((($this->part_size + $downloaded) / $total_down) * 100)
            : 0
            ;

        // don't write 100% to avoid confusing the user (99% could be "download finished but uncompressing)
        if($download_status<100)
        {
            $content = $download_status . 
                ";" . 
                $this->formatBytes($this->part_size + $downloaded) . 
                "/" . 
                $this->formatBytes($this->part_size + $total_down)
                ;

            file_put_contents($this->download_status_file, $content);
        }
    }

    public function pause_download($isbn)
    {
        $uncompress_status_file = $this->ci->config->item('products_folder') . $isbn . '/uncompress_status.txt';
        $pausing_file = $this->ci->config->item('products_folder') . $isbn . '/pausing.txt';

        // if uncompressing, no pause
        if(file_exists($uncompress_status_file)) throw new Exception('Download couldn\'t be paused. Uncompression in process');

        // write file 
        return touch($pausing_file);
    }

    public function _write_callback($ch, $str)
    {
        $length = FALSE;

        if(!empty($this->pausing_file)&&!file_exists($this->pausing_file))
        {
            $length = fwrite($this->fp, $str);
        }

        return $length;
    }


	private function formatBytes($size, $precision = 2)
	{
        $fmt = '0k';

        if($size>0)
        {
            $base = log($size) / log(1024);
            $suffixes = array('', 'k', 'M', 'G', 'T');   

            $fmt = round(pow(1024, $base - floor($base)), $precision) . $suffixes[floor($base)];
        }

        return $fmt;
	}

    public function set_markbook_address($isbn)
    {
        log_message('info', 'set_markbook_address: ' . $isbn);

        $this->ci->load->helper('url');

        $manifest_file = $isbn . '/sources/imsmanifest.xml';
        $config_tiger = $isbn . '/core/js/config.js';

        $full_path = $this->ci->config->item('products_folder') . $manifest_file;
        $full_path_tiger = $this->ci->config->item('products_folder') . $config_tiger;

        $markbook_address = '/slave/index.php/markbook/index/';
        // detect if it is a pulse or ct book
        if(file_exists($full_path))
        {
            log_message('info', $full_path . ' found');

            $simple_xml = simplexml_load_file($full_path);

            $attributes = $simple_xml->organizations->organization->metadata->Markbook->attributes();
            $att = 'originalUrl';

            if(!empty($attributes->$att))
            {
                $simple_xml->organizations->organization->metadata->Markbook->attributes()->$att = $simple_xml->organizations->organization->metadata->Markbook;
            }
            else
            {
                $simple_xml->organizations->organization->metadata->Markbook->addAttribute($att,  $simple_xml->organizations->organization->metadata->Markbook);
            }

            $simple_xml->organizations->organization->metadata->Markbook = $markbook_address . $manifest_file . '/';
            return file_put_contents($full_path, $simple_xml->asXML())!==FALSE;
        }
        elseif(file_exists($full_path_tiger))
        {
            log_message('info', $full_path_tiger . ' found');

            return file_put_contents($full_path_tiger, 'config.alternative_STF_Server="' . $markbook_address . '";', FILE_APPEND);

        }
        elseif(!file_exists($full_path_tiger) && !file_exists($full_path))
        {
            log_message('info', $full_path . ' not found');

            // Procedimiento:
            // 1) Buscar si existe ebook/markBook.html y buscar STF_Server . Obtener ruta real y guardarla en algún lado (se puede escribir un portable.json en la raiz del libro (o portable.xml). (esto mismo se podría hacer con los libros con manifiesto)
            // 2) Abrir el archivo core/config.js y agregar la ruta alterada: ;config.alternative_STF_Server = "slave/markbook/..etc"

            // assume that it's a tiger or interface one.
            return FALSE;
        }
    }

    public function update_files($isbn)
    {
        $folder = $this->ci->config->item('products_folder') . $isbn . '/';

        log_message('info', 'Update files for: ' . $folder);

        if(is_dir($folder . 'ebook'))
        {
            $str = file_get_contents('templates/ebook/desktop-player.js');

            // add license to desktop-player.js
            file_put_contents($folder . 'ebook/desktop-player.js', sprintf($str, $isbn));

        }

        // log_message('info', 'update_files: ' . $isbn);

        // Uncomment to add this config value (currenty not necessary)
        // if exists a particular config, then it's a pulse package and the config must be modified
        // $full_path = $this->ci->config->item('products_folder') . $isbn . '/sources/i2c_editor/core/controllers/config.js';
        // if(file_exists($full_path))
        // {
        //     file_put_contents($full_path, ';config.forceOpenFeedback = true;', FILE_APPEND);
        // }
    }
}
