<?php

class Slave_sams {

    public function __construct()
    {
        $this->ci =& get_instance();
    }

    // gets a local sid, no matter if it is a valid one
    public function get_local_sid($sams_id = NULL)
    {
        $this->ci->load->helper('cookie');
        $this->ci->load->helper('string');
        $this->ci->load->library('sams');
        $this->ci->load->model('session_model');

        $SID = get_cookie('MEC_WS_SESSIONID');

        // if there is no cookie, get the last one from sessions table
        if(empty($SID))
        {
            if(!empty($sams_id))
            {
                $sess = $this->session_model->get_by(array('sams_id' => $sams_id));

                if(!empty($sess))
                {
                    $SID = $sess->row()->session_id;
                }
            }

            if(empty($SID))
            {
                $SID = 'local-'.random_string('alnum', 26);
            }

            $this->ci->sams->set_cookie($SID);
        }

        return $SID;
    }

/*
 * 
 * name: authenticate_online
 * @param 
 * @return a session for an authenticated user online and offline
 */ 
    public function authenticate_online()
    {
        $this->ci->load->library('sams');
        $this->ci->load->model(array('session_model','user_model'));

        $local_SID = $this->get_local_sid();

        // se podria agregar validacion para ver si la sesion local esta autenticada online

        $SID = $this->ci->sams->get_valid_sid();

        if(empty($SID))
        {
            log_message('error', 'Failed while getting online session');
            throw new Exception('Failed while getting online session');
        }

        if($local_SID!=$SID)
        {
            log_message('info', 'Session differs: ' . 'SID: ' . $SID . ' - Local SID: ' . $local_SID);
            $this->ci->session_model->replace($local_SID, $SID);
        }

        $user = $this->ci->user_model->get_by_sid($SID);

        if(empty($user)) 
        {
            log_message('error', 'User session not found: ' . 'SID: ' . $SID . ' - Local SID: ' . $local_SID);
            throw new Exception('User session not found');
        }

        // is the user logged in server?
        $this->ci->sams->login(array(
            'SID' => $SID,
            'username' => $user->username,
            'password' => $user->password
        ));

        $session = new StdClass;
        $session->sams_id = $user->sams_id;
        $session->session_id = $SID;

        return $session;
    }

    public function sync_products($session = NULL)
    {
        $this->ci->load->library(array('sams_soap','package_downloader'));
        $this->ci->load->model(array('product_model', 'user_model'));

        if(empty($session))
        {
            $session = $this->authenticate_online();
        }

        $SID = $session->session_id;

        $soap_products = $this->ci->sams_soap->get_active_products($SID);

        // create folders, download icons, 
        $this->ci->package_downloader->update_products($soap_products);

        // insert products in database
        $this->ci->product_model->insert_from_soap($soap_products);

        log_message('info', $this->ci->product_model->db->last_query());

        // associate product with user
        $this->ci->user_model->update_products($session->sams_id, $soap_products);

        log_message('info', $this->ci->user_model->db->last_query());

        return $this->ci->product_model->get_by_user($session->sams_id, array(
            'prefix_local_path' => $this->ci->config->item('products_folder')
        ));
    }

    public function sync_requests(&$output, $session = NULL)
    {
        $this->ci->load->model('request_model');
        $this->ci->load->library('curl_com');

        if(empty($session))
        {
            $session = $this->authenticate_online();
        }

        $SID = $session->session_id;

        $start = microtime(TRUE);

        $datetime = $this->ci->curl_com->get(array(
            'resource' => $this->ci->config->item('server_time'),
            'format' => 'json'
        ));

        $latency = microtime(TRUE) - $start;

        $remote = strtotime($datetime['datetime']);

        // Cristian's algorithm applied to objects to be uploaded
        // local to remote
        $local = strtotime('now');
        $diff = ($local + ($latency / 2)) - $remote; 
        $s = $diff>0 ? '-' : '+';
        $diff = abs((int)$diff);

        $requests = $this->ci->request_model->get_many_by(array(
            'sams_id' => $session->sams_id,
            'synced_at IS NULL' => NULL
        ));

        $output = array(
            'max_attempts' => 0,
        );

        foreach($requests as $r)
        {
            $created_at = \DateTime::createFromFormat('Y-m-d H:i:s', $r->created_at);
            $created_at->modify($s.$diff.' seconds');

            $ch = curl_init($r->url . '&timestamp=' . $created_at->getTimestamp() . '&check_attempts=1');
            curl_setopt($ch, CURLOPT_COOKIE, 'MEC_WS_SESSIONID='.$SID);
            curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 5);    
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            $raw_response = curl_exec($ch);
            $response = json_decode($raw_response, TRUE);

            $info = curl_getinfo($ch);
            log_message('info', 'Curl: ' . $info['url']);
            log_message('info', 'Response sync requests: ' . $raw_response);

            if(curl_errno($ch)==0&&!empty($raw_response))
            {

                preg_match('/error.*max.*attempts/i', $raw_response, $matches);
                if(!empty($matches)) $output['max_attempts'] ++;

                $this->ci->request_model->update($r->id, array(
                    'synced_at' => date('Y-m-d H:i:s')
                ));
            }
        }
    }
}
