<?php

class Updater {

    public function __construct()
    {
        $this->ci =& get_instance();
        $this->remote_path = $this->ci->config->item('url_updates');
        $this->local_path = 'updates' . DIRECTORY_SEPARATOR;
        $this->base = '..';

        if(!is_dir($this->local_path))
        {
            if(!mkdir($this->local_path))
            {
                throw new Exception('Folder "' . $this->local_path . '" couldn\'t be created.');
            }
        }
    }

    public function check_and_update()
    {
        $count_updated = 0;

        $files = array(
            // TODO move destination value to a config item
            array('file' => 'portable.zip', 'destination' => $this->base),
        );

        // casos: 
        // no hay nada
        // existe un md5 (que se corresponde al ultimo zip descargado): current_i2c_api.zip.md5 y ningún zip
        // existe un md5 (que se corresponde al ultimo zip descargado): current_i2c_api.zip.md5 y un zip.part desactualizado
        // existe un md5 (que se corresponde al ultimo zip descargado): current_i2c_api.zip.md5 y un zip.part actualizado
        // existe un md5 (que se corresponde al ultimo zip descargado): current_i2c_api.zip.md5 y un zip
        //
        foreach($files as $file_config)
        {
            $f = $file_config['file'];
            $destination = $file_config['destination'];

            $current_local_md5_filename = $this->local_path . 'current_' . $f . '.md5';
            $local_md5_filename = $this->local_path . $f . '.md5';
            $remote_md5_filename = $this->remote_path . $f . '.md5';

            $local_zip_filename = $this->local_path . $f;
            $remote_zip_filename = $this->remote_path . $f;

            // TODO: verificar que no exista un zip.part o un zip nuevos (evitando update simultaneo)

            // download md5
            $fp = fopen($local_md5_filename, 'w');

            if (!$fp) throw new Exception('File can\'t be created');

            $fp_curl_log = fopen(APPPATH . 'logs/curl.log', 'w');

            $ch = curl_init($remote_md5_filename);

            curl_setopt($ch, CURLOPT_VERBOSE, 1);
            curl_setopt($ch, CURLOPT_STDERR, $fp_curl_log);
            curl_setopt($ch, CURLOPT_FILE, $fp);

            $result = curl_exec($ch);

            fclose($fp);
            fclose($fp_curl_log);

            log_message('info', 'Updater, md5 downloaded');

            // end download md5
            if(!$result)
            {
                $msg = 'MD5 file couldn\'t be downloaded: ' . curl_error($ch);
                curl_close($ch);

                log_message('error', $msg);

                throw new Exception($msg);
            }

            curl_close($ch);

            $md5 = file_get_contents($local_md5_filename);

            log_message('info', 'Getting MD5 contents ('.$local_md5_filename.'):' . $md5);

            $current_md5 = '';

            // does exist current md5 version ?
            if(file_exists($current_local_md5_filename))
            {
                $current_md5 = file_get_contents($current_local_md5_filename);
                log_message('info', 'Getting current MD5 contents('.$current_local_md5_filename.'):' . $current_md5);
            }

            log_message('info', 'Comparing MD5 files: ' . $md5 . ' vs ' . $current_md5);

            if($current_md5 != $md5)
            {
                log_message('info', 'Updater, md5 is different');

                if(!rename($local_md5_filename, $current_local_md5_filename))
                {
                    throw new Exception('MD5 File couldn\'t be saved');
                }

                // download zip
                $part_filename = $local_zip_filename.'.part';

                $fp = fopen($part_filename, 'w');
                if (!$fp) throw new Exception('File can\'t be created');

                $fp_curl_log = fopen(APPPATH . 'logs/curl.log', 'a');

                $ch = curl_init($remote_zip_filename);

                curl_setopt($ch, CURLOPT_VERBOSE, 1);
                curl_setopt($ch, CURLOPT_STDERR, $fp_curl_log);
                curl_setopt($ch, CURLOPT_FILE, $fp);

                $result = curl_exec($ch);

                fclose($fp);
                fclose($fp_curl_log);
                curl_close($ch);

                $md5_part = md5_file($part_filename);

                if(!$result)
                {
                    throw new Exception('ZIP file couldn\'t be downloaded: ' . curl_error($ch));
                }

                // verify file
                if(!(substr($md5, 0, 32)==substr($md5_part, 0, 32))) 
                {
                    throw new Exception('File is corrupted');
                }

                // rename part to zip
                if(!rename($part_filename, $local_zip_filename))
                {
                    throw new Exception('File couldn\'t be saved');
                }

                log_message('info', 'Updater, uncompressing zip');

                // extract zip
                $zip = new ZipArchive;
                if($zip->open($local_zip_filename)!==TRUE)
                {
                    throw new Exception('Zip file couldn\'t be opened');
                }

                $zip->extractTo($destination);
                $zip->close();

                log_message('info', 'Updater, uncompress ok');

                $count_updated++;
            }

        }

        log_message('info', 'Updater, files: ' . count($files) . ' - count_updated: ' . $count_updated);

        return count($files)==$count_updated;
    }

}
