<?php defined('BASEPATH') OR exit('No direct script access allowed');

class Migration_Check_db extends CI_Migration {
    public function up()
    {
        $this->load->dbforge();

        // the original schema has a db with a tabla named portable_requests.
        // this table should be dropped and recreated again and create the rest of the tables
        //
        if(!$this->db->table_exists('portable_users'))
        {
            // trigger a database process to start with the correct schema

            // drop portable_users if exists
            $this->dbforge->drop_table('portable_requests');

            // create the rest
            $this->dbforge->add_field('sams_id INTEGER');
            $this->dbforge->add_field('username VARCHAR(255)');
            $this->dbforge->add_field('password VARCHAR(50)');
            $this->dbforge->add_key('sams_id', TRUE);
            $this->dbforge->add_key('username');
            $this->dbforge->create_table('portable_users');

            $this->dbforge->add_field('session_id VARCHAR(50)');
            $this->dbforge->add_field('sams_id INTEGER');
            $this->dbforge->add_field('expires_at TIMESTAMP');
            $this->dbforge->add_key('session_id', TRUE);
            $this->dbforge->create_table('portable_sessions');
            $this->db->query('ALTER TABLE portable_sessions ADD FOREIGN KEY (sams_id) REFERENCES portable_users(sams_id) ON DELETE CASCADE ON UPDATE CASCADE');

            $this->dbforge->add_field('isbn VARCHAR(100) NOT NULL');
            $this->dbforge->add_field('image VARCHAR(255)');
            $this->dbforge->add_field('title VARCHAR(255)');
            $this->dbforge->add_field('zip_file VARCHAR(255)');
            $this->dbforge->add_field('version INTEGER(1) DEFAULT 0');
            $this->dbforge->add_field('downloaded_version INTEGER(1) DEFAULT 0');
            $this->dbforge->add_key('isbn', TRUE);
            $this->dbforge->create_table('portable_products');

            $this->dbforge->add_field('id INTEGER AUTO_INCREMENT');
            $this->dbforge->add_field('sams_id INTEGER');
            $this->dbforge->add_field('url TEXT');
            $this->dbforge->add_field('created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP');
            $this->dbforge->add_field('modified_at TIMESTAMP NULL');
            $this->dbforge->add_field('synced_at TIMESTAMP NULL');
            $this->dbforge->add_key('id', TRUE);
            $this->dbforge->create_table('portable_requests');
            $this->db->query('ALTER TABLE portable_requests ADD FOREIGN KEY (sams_id) REFERENCES portable_users(sams_id) ON DELETE CASCADE ON UPDATE CASCADE');

            $this->dbforge->add_field('sams_id INTEGER');
            $this->dbforge->add_field('isbn VARCHAR(100) NOT NULL');
            $this->dbforge->add_key('sams_id', TRUE);
            $this->dbforge->add_key('isbn', TRUE);
            $this->dbforge->create_table('portable_products_x_users');
            $this->db->query('ALTER TABLE portable_products_x_users ADD FOREIGN KEY (sams_id) REFERENCES portable_users(sams_id) ON DELETE CASCADE ON UPDATE CASCADE');
            $this->db->query('ALTER TABLE portable_products_x_users ADD FOREIGN KEY (isbn) REFERENCES portable_products(isbn) ON DELETE CASCADE ON UPDATE CASCADE');

        }

    }

    public function down()
    {
        // this migration shouldn't never be downgraded 
    }
}
